@@ -10,12 +10,13 @@ import pandas as pd
 
 
 """TKINTER PART"""
-"""
+
 import tkinter as tk
 
 from tkinter import LEFT
 
 root = tk.Tk()
+root.withdraw()
 
 
 WINDOW_SIZE=[1366,768]
@@ -29,7 +30,7 @@ embed.grid(columnspan = (600), rowspan = 500) # Adds grid
 
 os.environ['SDL_WINDOWID'] = str(embed.winfo_id())
 os.environ['SDL_VIDEODRIVER'] = 'windib'
-"""
+
 
 """TKINTER END PART"""
 
@@ -60,7 +61,7 @@ screen=initialize_pg()
 
 """TKINTER PART"""
 
-"""
+
 def open_file(): 
     file = askopenfile(mode ='r', filetypes =[('Python Files', '*.py')]) 
     filename=""
@@ -85,7 +86,8 @@ def run_script_from_file():
 
 global loaded_filename
 
-def open_xlsx_file(): 
+def open_xlsx_file(*args): 
+    table=args[0]
     file = askopenfile(mode ='r', filetypes =[('Excel files', '*.xlsx')]) 
     filename=""
     if file is not None: 
@@ -118,46 +120,42 @@ from tkinter.filedialog import askopenfile,asksaveasfile
 
 root.update()
 
-"""
+
 """TKINTER END PART"""
 
 
 class Label:
-    def __init__(self,pos,text,color,font="Cambria",fontsize=20,max_text_length=None):
-        self.pos=pos
+    def __init__(self,text,color,pos=[0,0],relative_pos=[0,0],font_type="Cambria",font_size=20,max_text_length=None,visible=True):
         self.text=text
         self.color=color
-        self.font=font
-        self.fontsize=fontsize
-        self.myfont = pygame.font.SysFont(self.font, self.fontsize)
+        self.pos=pos
         self.max_text_length=max_text_length
-        self.selected=False
-        
-    def draw(self):
-       
-        #print("Pixels",self.myfont.size(self.text))
-        self.shown_text=self.text
+        self.font_type=font_type
+        self.font_size=font_size
+        self.relative_pos=relative_pos
+        self.myfont = pygame.font.SysFont(self.font_type, self.font_size)
+        self.lbl=self.myfont.render(self.text, self.font_size, self.color)
+        self.selected=False #dbtable -> shows just beginning of string
+        self.visible=visible
         
+    def draw(self):       
+        self.shown_text=self.text    
         if self.max_text_length is not None:
-            #print("HEUREKA",self.max_text_length)
-            #print(self.shown_text)
+            #print(self.max_text_length,self.shown_text)
+        
             self.text_length=self.myfont.size(self.shown_text)[0]
-            #print("text_length",self.text_length,self.myfont.size(self.shown_text))
             while self.text_length>self.max_text_length:
-                #print(self.text_length)
                 if self.selected:
                     self.shown_text=self.shown_text[1:]
                 else:
-                    self.shown_text=self.shown_text[:-1]
-                
+                    self.shown_text=self.shown_text[:-1]              
                 self.text_length=self.myfont.size(self.shown_text)[0]
-            
-        
-        try:
-            lbl = self.myfont.render(self.shown_text, 1, self.color)
-        except pygame.error as e:
-            lbl = self.myfont.render("", 1, self.color)
-        screen.blit(lbl, (self.pos[0], self.pos[1]))
+        if self.visible:
+            try:
+                self.lbl=self.myfont.render(self.shown_text, self.font_size, self.color)
+            except pygame.error as e:
+                self.lbl = self.myfont.render("", 1, self.color)
+            screen.blit(self.lbl, (self.pos[0], self.pos[1]))
            
 def refresh(pgwidgets):
     screen.fill(bg_color)
@@ -173,19 +171,18 @@ def refresh(pgwidgets):
 
 class DraggableRect:
     def __init__(self,pos,size,color,draggable=True):
-        self.x=pos[0]
-        self.y=pos[1]
-        self.dx=size[0]
-        self.dy=size[1]
+        self.pos=pos
+        self.size=size
         self.color=color
         self.draggable=draggable
         self.selected=False
+        self.visible=True
         
     def draw(self): 
-        pygame.draw.rect(screen,self.color,[self.x,self.y,self.dx,self.dy])    
+        pygame.draw.rect(screen,self.color,[self.pos[0],self.pos[1],self.size[0],self.size[1]])    
     
     def is_point_in_rectangle(self,pos):
-        if self.x<pos[0] and pos[0]<self.x+self.dx and self.y<pos[1] and pos[1]<self.y+self.dy:
+        if self.pos[0]<pos[0] and pos[0]<self.pos[0]+self.size[0] and self.pos[1]<pos[1] and pos[1]<self.pos[1]+self.size[1]:
             return(True)
         else:
             return(False)
@@ -217,13 +214,13 @@ class Cell(DraggableRect):
     def __init__(self,pos,size,color,coor=[0,0]):
         super().__init__(pos,size,color,draggable=False)
         self.coor=coor
-        self.label=Label([pos[0]+2,pos[1]+4],"",(0,0,0),font="Calibri",fontsize=15,max_text_length=size[0]-1)
+        self.label=Label("",(0,0,0),[pos[0]+2,pos[1]+4],font_type="Calibri",font_size=15,max_text_length=size[0]-1)
                 
     def draw(self):
         if not self.selected:
             super().draw()
         else:
-            pygame.draw.rect(screen,self.color,[self.x,self.y,self.dx,self.dy])              
+            pygame.draw.rect(screen,self.color,[self.pos[0],self.pos[1],self.size[0],self.size[1]])              
         self.label.draw()              
 
 class Table:
@@ -233,7 +230,7 @@ class Table:
         self.rows=rows
         self.cols=cols
         self.margin=margin
-        self.table_size=[(self.cell_size[0]+self.margin)*self.cols,(self.cell_size[1]+self.margin)*self.rows]
+        self.table_size=[(self.cell_size[0]+self.margin)*self.cols,(self.cell_size[1]+self.margin)*(self.rows+1)]
         self.frame_cell=Cell(self.pos,self.table_size,(212,212,212))
         self.table_cells=[]
         self.initialize_cells()
@@ -241,7 +238,12 @@ class Table:
         self.df=None
         
     def initialize_cells(self):
-        for i in range(self.rows):
+        for j in range(self.cols):
+            i=0
+            new_pos=[self.pos[0]+j*self.cell_size[0]+j*self.margin,self.pos[1]+i*self.cell_size[1]+i*self.margin]
+            self.table_cells.append(Cell(new_pos,self.cell_size,(230,230,230),coor=[i,j]))
+        
+        for i in range(1,self.rows+1):
             for j in range(self.cols):
                 new_pos=[self.pos[0]+j*self.cell_size[0]+j*self.margin,self.pos[1]+i*self.cell_size[1]+i*self.margin]
                 self.table_cells.append(Cell(new_pos,self.cell_size,(255,255,255),coor=[i,j]))
@@ -255,9 +257,12 @@ class Table:
         #selected cell
         if self.selected_cell_index is not None:
             cell=self.table_cells[self.selected_cell_index]
-            pygame.draw.rect(screen,(33,115,70),[cell.x-2,cell.y-2,cell.dx+3,cell.dy+3],2) 
-            pygame.draw.rect(screen,(255,255,255),[cell.x+cell.dx-3,cell.y+cell.dy-3,6,6],2) 
-            pygame.draw.rect(screen,(33,115,70),[cell.x+cell.dx-2,cell.y+cell.dy-2,5,5]) 
+            pygame.draw.rect(screen,(33,115,70),[cell.pos[0]-2,cell.pos[1]-2,cell.size[0]+3,cell.size[1]+3],2) 
+            pygame.draw.rect(screen,(255,255,255),[cell.pos[0]+cell.size[0]-3,cell.pos[1]+cell.size[1]-3,6,6],2) 
+            pygame.draw.rect(screen,(33,115,70),[cell.pos[0]+cell.size[0]-2,cell.pos[1]+cell.size[1]-2,5,5]) 
+        
+        pygame.draw.rect(screen,(0,0,0),[self.pos[0]-1,self.pos[1]-1]+self.table_size,3)
+        
             
             
     def is_clicked(self,pos):
@@ -320,14 +325,26 @@ class Table:
         pass
                 
                 
-    def update_data(self,df):
+    def update_data(self,df, rows = None):
         """data layer"""  
         self.df=df
+        self.table_cells=[]
+        self.initialize_cells()
+        if rows is None:
+            rows = df.shape[0]
+        print("updatedata",self.df)
         
         list1=df.values.tolist()
-        for i in range(len(list1)):
+        
+        column_names=df.columns
+        i=0
+        for j in range(len(column_names)):
+            cell_index=self.find_cell_index(0,j)
+            if cell_index is not None:
+                self.table_cells[cell_index].label.text=str(column_names[j])     
+        for i in range(min(len(list1), rows)):
             for j in range(len(list1[i])):
-                cell_index=self.find_cell_index(i,j)
+                cell_index=self.find_cell_index(i+1,j) #skipping header -> +1
                 if cell_index is not None:
                     self.table_cells[cell_index].label.text=str(list1[i][j])           
 
@@ -349,10 +366,10 @@ class ButtonImage(DraggableRect):
     def run_function(self):
         self.function()
 
-"""
+
 def save_df(table1):
     table1.df.to_excel("name.xlsx")
-"""    
+  
 
 class RadioButton(DraggableRect):
     def __init__(self,pos,text,radio_group,selected=False):
@@ -362,8 +379,19 @@ class RadioButton(DraggableRect):
         self.text=text
         self.radio_group=radio_group
         self.selected=selected
-        self.img_empty=pygame.image.load("radio_empty.png")
-        self.img_full=pygame.image.load("radio_full.png")
+                
+        try:
+            self.img_empty = pygame.image.load('radio_empty.png')
+        except:
+            self.img_empty = pygame.image.load('img//radio_empty.png')
+            
+        try:
+            self.img_full = pygame.image.load('radio_full.png')
+        except:
+            self.img_full = pygame.image.load('img//radio_full.png')
+            
+        #self.img_empty=pygame.image.load("radio_empty.png")
+        #self.img_full=pygame.image.load("radio_full.png")
         self.rescale()
 
     def rescale(self):
@@ -385,8 +413,16 @@ class Checkbox(DraggableRect):
         self.pos=pos
         self.text=text
         self.selected=selected
-        self.img_empty=pygame.image.load("checkbox_empty.png")
-        self.img_full=pygame.image.load("checkbox_full.png")
+        try:
+            self.img_empty = pygame.image.load('checkbox_empty.png')
+        except:
+            self.img_empty = pygame.image.load('img//checkbox_empty.png')
+            
+        try:
+            self.img_full = pygame.image.load('checkbox_full.png')
+        except:
+            self.img_full = pygame.image.load('img//checkbox_full.png')
+            
         self.rescale()
 
     def rescale(self):
@@ -405,16 +441,58 @@ class TextArea(DraggableRect):
         self.pos=pos
         self.size=size
         self.text=text
-        self.label=Label([pos[0]+2,pos[1]+4],self.text,(0,0,0),font="Calibri",fontsize=15,max_text_length=size[0]-1)
+        self.label=Label(self.text,(0,0,0),[pos[0]+2,pos[1]+4],font_type="Calibri",font_size=15,max_text_length=size[0]-1)
         self.border_color=border_color
         self.color=color
+        self.fit2label()
+
+    def blit_text(self, surface, text, color=pygame.Color('black')):
+        words = [word.split(' ') for word in self.label.text.splitlines()]
+        space = self.label.myfont.size(' ')[0]
+        max_height = self.size[1]-10
+        x, y = self.label.pos
+        for line in words:
+            for word in line:
+                word_surface = self.label.myfont.render(word, 0, color)
+                word_width, word_height = word_surface.get_size()
+                if x + word_width >= self.label.max_text_length + self.label.pos[0]:
+                    x = self.label.pos[0]
+                    y += word_height
+                    if y+word_height >= self.label.pos[1] + max_height:
+                        
+                        break
+                    
+        
+                surface.blit(word_surface, (x, y))
+                x += word_width + space
+            x = self.label.pos[0]
+            y += word_height
+            if y >= self.label.pos[1] + max_height:
+                break
         
     def draw(self):
         super().draw()
-        pygame.draw.rect(screen,self.border_color,[self.x,self.y,self.dx,self.dy],1)  
-        self.label.draw()  
-        
-    
+        pygame.draw.rect(screen,self.border_color,[self.pos[0],self.pos[1],self.size[0],self.size[1]],1)
+        self.label.draw = self.blit_text
+        self.label.draw(screen, self.label.text)
+
+    def fit2label(self):
+        words = self.text.split()
+        final_text = ""
+        line = ""
+        for word in words:
+            if line:
+                if self.label.myfont.size(line + " " + word)[0] > self.size[0] + 3:
+                    final_text += line + "\n"
+                    line = word
+                else:
+                    line += " " + word
+            # empty line
+            else:
+                line += word
+        if line:
+            final_text += line + "\n"
+        self.label.text = final_text
 
 
 class Button(DraggableRect):
@@ -425,17 +503,26 @@ class Button(DraggableRect):
         self.border_color=border_color
         self.color=color
         self.text=text
-        self.label=Label([pos[0]+2,pos[1]+4],self.text,(0,0,0),font="Calibri",fontsize=15,max_text_length=size[0]-1)
+        self.label=Label(self.text,(0,0,0),[pos[0]+2,pos[1]+4],font_type="Calibri",font_size=15,max_text_length=size[0]-1)
         self.function=function
-        
+        self.is_clicked=False
         
     def draw(self):
-        super().draw()
-        pygame.draw.rect(screen,self.border_color,[self.x,self.y,self.dx,self.dy],1)  
+        if self.is_clicked:
+            pygame.draw.rect(screen,(180,180,180),[self.pos[0],self.pos[1],self.size[0],self.size[1]]) 
+        else:
+            super().draw()
+        pygame.draw.rect(screen,self.border_color,[self.pos[0],self.pos[1],self.size[0],self.size[1]],1)  
         self.label.draw() 
     
-    def run_function(self):
-        self.function()
+    def run_function(self,*args):
+        if len(args)>0:
+            self.function(args)
+        else:
+            self.function()
+        
+
+    
 
 
 
@@ -447,28 +534,55 @@ class ComboBox(DraggableRect):
         self.values = values
         self.text = text
         self.border_color = border_color
-        self._labels = self._getLabels()
+        self._labels = []
         self._is_rolled = False
+        if text:
+            self.cell = Cell(self.pos, self.size, (212,212,212), coor=[0, 0])
+            self.cell.label.text = text
+            self.cell.label.pos[0]=self.cell.label.pos[0]+1 #2 pixels from border
+
+    def draw(self):
+        if hasattr(self, "text"):
+            pygame.draw.rect(screen,self.color,self.cell.pos+self.cell.size)    
+            self.cell.draw()         
+            pygame.draw.rect(screen,(0,0,0),self.cell.pos+self.cell.size,2)
+            pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-15,self.pos[1]+self.size[1]/2],[self.pos[0]+self.size[0]-10,self.pos[1]+self.size[1]/2+5],3)
+            pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-10,self.pos[1]+self.size[1]/2+5],[self.pos[0]+self.size[0]-5,self.pos[1]+self.size[1]/2],3)
+            pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-18,self.pos[1]],[self.pos[0]+self.size[0]-18,self.pos[1]+self.size[1]],1)
+            
+        if self._is_rolled:
+            for label, value in zip(self._labels, self.values):
+                label.label.text = value
+                label.draw()
+                
 
     def roll(self):
         if not self._is_rolled:
             self._is_rolled = not self._is_rolled
-            self.dy = self.size[1] * (len(self.values) + 1)
-            self.draw()
-            for label in self._labels:
-                label.draw()
+            self._labels = self._getLabels()
+            self._dy = self.size[1] * (len(self.values) + 1)        
         else:
             self._is_rolled = not self._is_rolled
-            self.x = self.pos[0]
-            self.y = self.pos[1]
-            self.dx = self.size[0]
-            self.dy = self.size[1]
-            self.draw()
+    
+    
+    def choose(self, pos):
+        if self._is_rolled:
+            length = pos[1] - self.pos[1]
+            idx = abs(length // self.size[1])
+            print(idx)
+            if idx == 0:
+                self.roll()
+            else:
+                self.cell.label.text = self.values[idx - 1]
+                self.roll()
+        elif self.is_point_in_rectangle(pos):
+            self.roll()
+    
 
     def _getLabels(self):
         tmp = []
         for i in range(len(self.values)):
-            tmp.append(Label([self.x + 2, self.y * (i + 2) + 4], self.values[i], (0,0,0), fontsize=self.size[1] - 2, max_text_length=self.size[0]-1))
+            tmp.append(Cell([self.pos[0], self.pos[1] + self.size[1] * (i + 1)], self.size, (212,212,212), coor=[0, i + 1]))
         return (tmp)
 
 
@@ -490,7 +604,7 @@ def main_program_loop(pgwidgets,table1):
     new_select_possible=True
     direction_list=[0,0,0,0]
     selected=None
-    
+
     while not done:  
         try:   
             for event in pygame.event.get():
@@ -500,6 +614,7 @@ def main_program_loop(pgwidgets,table1):
                     if new_select_possible: #Because of popping from the list
                         pos = pygame.mouse.get_pos()
                         
+                        
                         """
                         for i,rect in enumerate(rects):
                             if pos[0]>rect.x and pos[0]<rect.x+rect.dx and pos[1]>rect.y and pos[1]<rect.y+rect.dy:
@@ -509,9 +624,36 @@ def main_program_loop(pgwidgets,table1):
                         """
                         for i,widget_type in enumerate(pgwidgets):
                             widget_type.click_element_function(pos)
+                            
+                        #Dynamic drawing of button,combo click
+                        for i,widget_type in enumerate(pgwidgets):
+                            for j,widget in enumerate(widget_type.elements):
+                                if type(widget)==Button:
+                                    if widget.is_point_in_rectangle(pos):
+                                        pgwidgets[i].elements[j].is_clicked=True
+                                        
+                                if type(widget)==ComboBox:
+                                    if widget._is_rolled and widget.pos[1] < pos[1] and pos[1] < widget.pos[1] + widget.size[1] * (len(widget.values) + 1):
+                                        widget.choose(pos)
+                                    elif not widget._is_rolled and widget.is_point_in_rectangle(pos):
+                                        widget.choose(pos)
+                            
+                            
                         
                                 
                 elif event.type == pygame.MOUSEBUTTONUP:
+                    pos = pygame.mouse.get_pos()
+                    
+                    
+                    #Dynamic drawing of button click
+                    for i,widget_type in enumerate(pgwidgets):
+                        for j,widget in enumerate(widget_type.elements):
+                            if type(widget)==Button:
+                                pgwidgets[i].elements[j].is_clicked=False
+                        
+                        
+                    
+                    
                     selected = None
                     new_select_possible=True
                 elif event.type == pygame.MOUSEMOTION:
@@ -576,13 +718,13 @@ def main_program_loop(pgwidgets,table1):
                 pygame.event.pump()
                 keys = pygame.key.get_pressed()
                 
-                """
+                
             try: #Handling tkinter and pygame loops
                 root.update()
             except tk.TclError as e:
                 sys.exit(0)
                 print(e)
-                """
+                
             pygame.display.flip()   
     
         except KeyboardInterrupt:
